#include	"..\DLL\d_iNES.h"

static	struct
{
	u8 Cmd, IRQenabled;
	u16_n IRQcounter;
	u8 CHR[8], PRG[4];
	u8 Mirror;
}	Mapper;

static	void	SaveMI (Ar128 MI)
{
	u8 x = 0;
	MI[x++] = Mapper.IRQcounter.b0;
	MI[x++] = Mapper.IRQcounter.b1;
	MI[x++] = Mapper.IRQenabled;
	MI[x++] = Mapper.Cmd;
}

static	void	LoadMI (const Ar128 MI)
{
	u8 x = 0;
	Mapper.IRQcounter.b0	= MI[x++];
	Mapper.IRQcounter.b1	= MI[x++];
	Mapper.IRQenabled	= MI[x++];
	Mapper.Cmd		= MI[x++];
	Sync();
}

static	void	HBlank (int Scanline, int Byte2001)
{
	if (Mapper.IRQenabled)
	{
		if (Mapper.IRQcounter.s0 <= 114)
			MP->IRQ();
		else	Mapper.IRQcounter.s0 -= 114;
	}
}

static	void	Sync (void)
{
	u8 x;
	for (x = 0; x < 8; x++)
		MP->SetCHR_ROM1(x,Mapper.CHR[x]);
	if (Mapper.PRG[0] & 0x40)
	{
		if (Mapper.PRG[0] & 0x80)
			MP->SetPRG_RAM8(0x6,0);
		else
		{
			MP->SetPRG_OB4(0x6);
			MP->SetPRG_OB4(0x1);
		}
	}
	else	MP->SetPRG_ROM8(0x6,Mapper.PRG[0]);
	MP->SetPRG_ROM8(0x8,Mapper.PRG[1]);
	MP->SetPRG_ROM8(0xA,Mapper.PRG[2]);
	MP->SetPRG_ROM8(0xC,Mapper.PRG[3]);
	MP->SetPRG_ROM8(0xE,-1);
	switch (Mapper.Mirror)
	{
	case 0:	MP->Mirror_V();		break;
	case 1:	MP->Mirror_H();		break;
	case 2:	MP->Mirror_S0();	break;
	case 3:	MP->Mirror_S1();	break;
	}
}

static	void	Write (int Bank, int Where, int What)
{
	switch (Bank & 0xE)
	{
	case 0x8:	Mapper.Cmd = What & 0xF;	break;
	case 0xA:	switch (Mapper.Cmd)
			{
			case 0x0:	Mapper.CHR[0] = What;	break;
			case 0x1:	Mapper.CHR[1] = What;	break;
			case 0x2:	Mapper.CHR[2] = What;	break;
			case 0x3:	Mapper.CHR[3] = What;	break;
			case 0x4:	Mapper.CHR[4] = What;	break;
			case 0x5:	Mapper.CHR[5] = What;	break;
			case 0x6:	Mapper.CHR[6] = What;	break;
			case 0x7:	Mapper.CHR[7] = What;	break;
			case 0x8:	Mapper.PRG[0] = What;	break;
			case 0x9:	Mapper.PRG[1] = What;	break;
			case 0xA:	Mapper.PRG[2] = What;	break;
			case 0xB:	Mapper.PRG[3] = What;	break;
			case 0xC:	Mapper.Mirror = What & 3;	break;
			case 0xD:	Mapper.IRQenabled = What;	break;
			case 0xE:	Mapper.IRQcounter.b0 = What;	break;
			case 0xF:	Mapper.IRQcounter.b1 = What;	break;
			}				break;
	case 0xC:	/* sound goes here */	break;
	case 0xE:	/* ditto */		break;
	}
}

static	void	UnloadMapper (void)
{
	iNES_UnloadROM();
}

static	void	InitMapper (const PMapperParam _MP, int IsHardReset)
{
	u8 x;
	MP = _MP;
	for (x = 0x8; x < 0x10; x++)
		MP->SetWriteHandler(x,Write);

	MP->SetPRG_ROM16(0x8,0);
	MP->SetPRG_ROM16(0xC,-1);

	MP->SetCHR_ROM8(0,0);

	iNES_InitROM();

	Mapper.Cmd = Mapper.IRQenabled = 0;
	Mapper.IRQcounter.s0 = 0;
	for (x = 0; x < 4; x++)	Mapper.PRG[x] = Mapper.CHR[x|0] = Mapper.CHR[x|4] = 0;
	Mapper.Mirror = 0;
	Sync();
}

const	TMapperInfo	MapperInfo_NES_JLROM =
{
	"NES-JLROM",
	-1,
	MS_Nearly,
	8192,
	InitMapper,
	UnloadMapper,
	HBlank,
	NULL,
	SaveMI,
	LoadMI,
	NULL,
	NULL
};