#include	"..\DLL\d_iNES.h"
#include	"..\Hardware\Sound\s_VRC7.h"

static	struct
{
	u8 IRQenabled, IRQtoggle, IRQlatch, IRQcount;
	u8 sndreg;
}	Mapper;

static	void	SaveMI (Ar128 MI)
{
	u8 x = 0, i;
			MI[x++] = Mapper.IRQenabled;
			MI[x++] = Mapper.IRQtoggle;
			MI[x++] = Mapper.IRQlatch;
			MI[x++] = Mapper.IRQcount;
			MI[x++] = Mapper.sndreg;
for (i = 0; i < 8; i++)	MI[x++] = VRC7sound.Instrument[0][i];
for (i = 0; i < 6; i++)	MI[x++] = VRC7sound.Chan1x[i];
for (i = 0; i < 6; i++)	MI[x++] = VRC7sound.Chan2x[i];
for (i = 0; i < 6; i++)	MI[x++] = VRC7sound.Chan3x[i];
}

static	void	LoadMI (const Ar128 MI)
{
	u8 x = 0, i;
			Mapper.IRQenabled		= MI[x++];
			Mapper.IRQtoggle		= MI[x++];
			Mapper.IRQlatch			= MI[x++];
			Mapper.IRQcount			= MI[x++];
			Mapper.sndreg			= MI[x++];
for (i = 0; i < 8; i++)	VRC7sound.Instrument[0][i]	= MI[x++];
for (i = 0; i < 6; i++)	VRC7sound.Chan1x[i]		= MI[x++];
for (i = 0; i < 6; i++)	VRC7sound.Chan2x[i]		= MI[x++];
for (i = 0; i < 6; i++)	VRC7sound.Chan3x[i]		= MI[x++];

for (x = 0; x < 6; x++){VRC7_LoadInstrument(x);
			VRC7_SoundWrite((u8)(0x10 + x),VRC7sound.Chan1x[x]);}
}

static	void	HBlank (int Scanline, int Byte2001)
{
	if (Mapper.IRQenabled)
	{
		if (Mapper.IRQcount == 0xFF)
		{
			Mapper.IRQcount = Mapper.IRQlatch;
			MP->IRQ();
		}
		else	Mapper.IRQcount++;
	}
}

static	void	PRG_Write_8 (int Bank, int Where, int What)
{
	Where |= (Where & 0x8) << 1;
	switch (Where & 0x10)
	{
	case 0x000:	MP->SetPRG_ROM8(0x8,What);	break;
	case 0x010:	MP->SetPRG_ROM8(0xA,What);	break;
	}
}

static	void	PRG_Write_9 (int Bank, int Where, int What)
{
	switch (Where & 0x30)
	{
	case 0x000:	MP->SetPRG_ROM8(0xC,What);	break;
	case 0x010:	Mapper.sndreg = What;		break;
	case 0x020:	MP->DbgOut("Write to $9020!");	break;
	case 0x030:	VRC7_SoundWrite(Mapper.sndreg,(u8)(What));
							break;
	}
}

static	void	PRG_Write_A (int Bank, int Where, int What)
{
	Where |= (Where & 0x8) << 1;
	switch (Where & 0x10)
	{
	case 0x000:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(0,What);
		else MP->SetCHR_RAM1(0,What);
		break;
	case 0x010:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(1,What);
		else MP->SetCHR_RAM1(1,What);
		break;
	}
}

static	void	PRG_Write_B (int Bank, int Where, int What)
{
	Where |= (Where & 0x8) << 1;
	switch (Where & 0x10)
	{
	case 0x000:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(2,What);
		else MP->SetCHR_RAM1(2,What);
		break;
	case 0x010:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(3,What);
		else MP->SetCHR_RAM1(3,What);
		break;
	}
}

static	void	PRG_Write_C (int Bank, int Where, int What)
{
	Where |= (Where & 0x8) << 1;
	switch (Where & 0x10)
	{
	case 0x000:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(4,What);
		else MP->SetCHR_RAM1(4,What);
		break;
	case 0x010:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(5,What);
		else MP->SetCHR_RAM1(5,What);
		break;
	}
}

static	void	PRG_Write_D (int Bank, int Where, int What)
{
	Where |= (Where & 0x8) << 1;
	switch (Where & 0x10)
	{
	case 0x000:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(6,What);
		else MP->SetCHR_RAM1(6,What);
		break;
	case 0x010:
		if (MP->CHR_ROM_Size) MP->SetCHR_ROM1(7,What);
		else MP->SetCHR_RAM1(7,What);
		break;
	}
}

static	void	PRG_Write_E (int Bank, int Where, int What)
{
	Where |= (Where & 0x8) << 1;
	switch (Where & 0x10)
	{
	case 0x000:	switch (What & 0x3)
			{
			case 0:	MP->Mirror_V();		break;
			case 1:	MP->Mirror_H();		break;
			case 2:	MP->Mirror_S0();	break;
			case 3:	MP->Mirror_S1();	break;
			}			break;
	case 0x010:	Mapper.IRQlatch = What;	break;
	}
}

static	void	PRG_Write_F (int Bank, int Where, int What)
{
	Where |= (Where & 0x8) << 1;
	switch (Where & 0x10)
	{
	case 0x000:	Mapper.IRQtoggle = What & 0x01;
			Mapper.IRQenabled = What & 0x02;
			if (Mapper.IRQenabled)
				Mapper.IRQcount = Mapper.IRQlatch;	break;
	case 0x010:	Mapper.IRQenabled = Mapper.IRQtoggle;	break;
	}
}

static	void	UnloadMapper (void)
{
	iNES_UnloadROM();
	KillOPL2();
}

static	void	MapperSnd (s16 *Buffer, int Len)
{
	GetVRC7Snd(Buffer,Len);
}

static	void	InitMapper (const PMapperParam _MP, int IsHardReset)
{
	MP = _MP;

	MP->SetWriteHandler(0x8,PRG_Write_8);
	MP->SetWriteHandler(0x9,PRG_Write_9);
	MP->SetWriteHandler(0xA,PRG_Write_A);
	MP->SetWriteHandler(0xB,PRG_Write_B);
	MP->SetWriteHandler(0xC,PRG_Write_C);
	MP->SetWriteHandler(0xD,PRG_Write_D);
	MP->SetWriteHandler(0xE,PRG_Write_E);
	MP->SetWriteHandler(0xF,PRG_Write_F);

	MP->SetPRG_ROM16(0x8,0);
	MP->SetPRG_ROM16(0xC,-1);

	if (MP->CHR_ROM_Size) MP->SetCHR_ROM8(0,0);
	else MP->SetCHR_RAM8(0,0);

	iNES_InitROM();
	Mapper.IRQenabled = Mapper.IRQtoggle = Mapper.IRQlatch = Mapper.IRQcount = 0;
	Mapper.sndreg = 0;
	CreateOPL2();
}

const	TMapperInfo	MapperInfo_085 =
{
	"Konami VRC7",
	85,
	MS_Full,
	8192,
	InitMapper,
	UnloadMapper,
	HBlank,
	NULL,
	SaveMI,
	LoadMI,
	MapperSnd,
	NULL
};